using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using System.Collections;

using Amazon1.com.amazonaws.ecs;

namespace Amazon1
{
    public partial class Form1 : Form
    {

        private string appId;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            appId = Properties.Settings.Default.ApplicationId;
        }

        private void settingButton_Click(object sender, EventArgs e)
        {
            using (Form2 idDialog = new Form2())
            {
                idDialog.ApplicationID = appId;

                if (idDialog.ShowDialog() == DialogResult.OK)
                {
                    appId = idDialog.ApplicationID;
                    Properties.Settings.Default.ApplicationId = appId;
                    Properties.Settings.Default.Save();
                }
            }
        }

        private void goButton_Click(object sender, EventArgs e)
        {
            try
            {
                string[] asinCodes = asinTextBox.Text.Split(',');
                string[] responseGroups = new string[] { "Medium" };
                ItemLookupResponse response = ItemLookup(asinCodes, responseGroups);
                if (response != null)
                    DisplayResponse(response);
           }
           catch (Exception ex)
           {
               MessageBox.Show(ex.ToString(), Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
           }
        }

        private ItemLookupResponse ItemLookup(string[] asinCodes, string[] responseGroups)
        {
            ItemLookupResponse result = null;
            using (AWSECommerceService service = new AWSECommerceService())
            {
                ItemLookup lookupOperation = new ItemLookup();
                lookupOperation.AWSAccessKeyId = appId;
                ItemLookupRequest lookupRequest = new ItemLookupRequest();
                lookupRequest.IdType = ItemLookupRequestIdType.ASIN;
                lookupRequest.ResponseGroup = responseGroups;
                lookupRequest.ItemId = asinCodes;
                lookupRequest.IdType = ItemLookupRequestIdType.EAN;
             
                lookupOperation.Request = new ItemLookupRequest[] { lookupRequest };
                result = service.ItemLookup(lookupOperation);
            }
            return result;
        }

        private void DisplayResponse(ItemLookupResponse response)
        {
            try
            {
                listView1.BeginUpdate();
                listView1.Items.Clear();
                listView1.Groups.Clear();

                ErrorCheck(response.OperationRequest.Errors);

                foreach (Items items in response.Items)
                {
                    ErrorCheck(items.Request.Errors);

                    if (items.Item != null)
                        foreach (Item item in items.Item)
                        {
                            ErrorCheck(item.Errors);
                            ListViewGroup group = new ListViewGroup(item.ASIN, item.ASIN);
                            listView1.Groups.Add(group);
                            
                            AddItem("", item, group);
                        }
                }
            }
            finally
            {
                listView1.EndUpdate();
            }
        }

        private void AddItem(string parentName, object response, ListViewGroup group)
        {
            foreach (PropertyInfo property in response.GetType().GetProperties())
                if (property.CanRead)
                {
                    string name = parentName + property.Name;
                    object value = property.GetValue(response, null);

                    if (value == null)
                        continue;
                    else if (value is string || value.GetType().IsValueType)
                        _addItem(name, value, group);
                    else if (!(value is IList))
                        AddItem(name + "/", value, group);
                    else if (property.Name == "SyncRoot")
                        continue;
                    else
                        foreach (object obj in (IList)value)
                            if (obj is string ||
                                obj is ValueType)
                                _addItem(name, obj, group);
                            else
                                AddItem(name + "/", obj, group);
                }
        }

        private void _addItem(string name, object value, ListViewGroup group)
        {
            string[] itemText = { name, value.ToString() };
            ListViewItem item = new ListViewItem(itemText, group);
            listView1.Items.Add(item);
        }

        private void ErrorCheck(ErrorsError[] errors)
        {
            if (errors != null)
            {
                string message = null;

                foreach (ErrorsError err in errors)
                    message += err.Message + "\r\n";

                throw new ApplicationException(message);
            }
        }
    }
}