using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using Amazon2.com.amazonaws.ecs;

namespace Amazon2
{
    public partial class Form1 : Form
    {
        private string appId;

        public Form1()
        {
            InitializeComponent();
        }

        private void goButton_Click(object sender, EventArgs e)
        {
            ItemSearchRequest request = new ItemSearchRequest();
            ItemSearchResponse response = null;

            request.SearchIndex = ((Store)storeComboBox.SelectedItem).Name;
            request.Keywords = keywordTextBox.Text;
            request.Sort = ((Content)sortComboBox.SelectedItem).Name;
            request.ResponseGroup = new string[] { "Small", "OfferSummary", "Images" };
            request.Count = "25";

            ItemSearch search = new ItemSearch();
            search.Request = new ItemSearchRequest[] { request };
            search.AWSAccessKeyId = appId;
            AWSECommerceService service = new AWSECommerceService();

            int reachCount = (int)numericUpDown1.Value;

            int getPage = 1;
            int totalPage = 0;
            int requestCount = reachCount;
            long lastTick = 0;

            List<Item> itemList = new List<Item>();

            try
            {
                do
                {
                    request.ItemPage = getPage++.ToString();

                    while (DateTime.Now.Ticks - lastTick < 10000000)
                        ;
                    lastTick = DateTime.Now.Ticks;

                    response = service.ItemSearch(search);
                    ParseResponse(response, itemList,
                        requestCount - itemList.Count);
                    requestCount =
                        Math.Min(int.Parse(response.Items[0].TotalResults),
                        reachCount);
                    totalPage = int.Parse(response.Items[0].TotalPages);

                } while (requestCount > itemList.Count && totalPage >= getPage);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, this.Text,  MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            string itemsText = "";

            foreach (Item current in itemList)
                itemsText += ItemHtml(current);

            string htmlText = GetPageHtml().Replace("<!-- contents -->", itemsText);

            webBrowser1.DocumentText = htmlText;

        }

        private string ItemHtml(Item myItem)
        {
            string itemUrl = myItem.DetailPageURL;
            string smallImageUrl = null;

            if (myItem.ImageSets != null)
                foreach (ItemImageSets imageSet in myItem.ImageSets)
                    foreach (ImageSet mySet in imageSet.ImageSet)
                    {
                        if (mySet.Category == "primary" &&
                            mySet.SmallImage != null)
                        {
                            smallImageUrl = string.Format(@"<a href=""{0}"" target=""_blank""><img src=""{1}""></a>", itemUrl, mySet.SmallImage.URL);
                        }
                    }

            string price = null;
            if (myItem.OfferSummary != null && myItem.OfferSummary.LowestNewPrice != null)
                price =
                    myItem.OfferSummary.LowestNewPrice.FormattedPrice;

            string html = string.Format(@"<p class=""leftimg"">{0}</p><p class=""righttext""><a href=""{1}"" target=""_blank"">{2}</a><br />{3}</p><p class=""clear"" />",
                smallImageUrl, itemUrl, myItem.ItemAttributes.Title,
                price);
            return html;
        }

        private void ParseResponse(ItemSearchResponse response, List<Item> itemList, int getCount)
        {
            ErrorCheck(response.OperationRequest.Errors);

            foreach (Items items in response.Items)
            {
                ErrorCheck(items.Request.Errors);

                if (items.Item != null)
                    foreach (Item item in items.Item)
                    {
                        if (getCount-- < 1)
                            break;

                        ErrorCheck(item.Errors);
                        itemList.Add(item);
                    }
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            appId = Properties.Settings.Default.ApplicationId;

            foreach (Store item in SearchIndex.All)
                storeComboBox.Items.Add(item);

            storeComboBox.SelectedIndex = 0;
        }

        private void ErrorCheck(ErrorsError[] errors)
        {
            if (errors != null)
            {
                string message = null;

                foreach (ErrorsError err in errors)
                    message += err.Message + "\r\n";

                throw new ApplicationException(message);
            }
        }

        private void storeComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            sortComboBox.Items.Clear();
            Store selectedStore = (Store)storeComboBox.SelectedItem;
            foreach (Content sort in selectedStore.Sorts)
                sortComboBox.Items.Add(sort);

            if (sortComboBox.Items.Count > 0)
                sortComboBox.SelectedIndex = 0;
        }

        private void settingButton_Click(object sender, EventArgs e)
        {
            using (Form2 idDialog = new Form2())
            {
                idDialog.ApplicationID = appId;

                if (idDialog.ShowDialog() == DialogResult.OK)
                {
                    appId = idDialog.ApplicationID;
                    Properties.Settings.Default.ApplicationId = appId;
                    Properties.Settings.Default.Save();
                }
            }

        }

        private string GetPageHtml()
        {
            using (StreamReader sr = new StreamReader(Path.Combine(Application.StartupPath, 
                "template.html"),Encoding.GetEncoding("shift_jis")))
            {
                return sr.ReadToEnd();
            }
        }
    }

    class SearchSort
    {
        public static readonly Content Empty =
            new Content("", "я͎wł܂B");
        public static readonly Content DateL =
            new Content("daterank", "t̐V̂");
        public static readonly Content MinAgeU =
            new Content("mfg-age-min", "ŏΏ۔N̏Ȃ̂");
        public static readonly Content MinAgeL = 
            new Content("-mfg-age-min", "ŏΏ۔N̑傫Ȃ̂");
        public static readonly Content Price1U =
            new Content("pricerank", "lï̂");
        public static readonly Content Price1L =
            new Content("-pricerank", "li̍̂");
        public static readonly Content PriceL =
            new Content("inverse-pricerank", "li̍̂");
        public static readonly Content Price2U =
            new Content("price", "lï̂");
        public static readonly Content Price2L =
            new Content("-price", "li̍̂");
        public static readonly Content RelDate1U =
            new Content("orig-rel-date", "̌Â̂");
        public static readonly Content RelDate1L =
            new Content("-orig-rel-date", "̐V̂");
        public static readonly Content RelDate2U =
            new Content("release-date", "̌Â̂");
        public static readonly Content RelDate2L =
            new Content("-release-date", "̐V̂");
        public static readonly Content SalesL =
            new Content("salesrank", "グ̍̂");
        public static readonly Content TitleU =
            new Content("titlerank", "^CgiABCj");
        public static readonly Content TitleL =
            new Content("-titlerank", "^Cg~iZYXj");

        public static readonly Content[] Blended = { Empty };
        public static readonly Content[] Baby = { SalesL, Price2U, Price2L, TitleU };
        public static readonly Content[] Books = { SalesL, Price1U, PriceL, TitleU, TitleL, DateL };
        public static readonly Content[] Classical = { SalesL, Price1U, Price1L, TitleU, TitleL, RelDate1U, RelDate1L };
        public static readonly Content[] DVD = { SalesL, Price1U, Price1L, TitleU, TitleL, RelDate1U, RelDate1L };
        public static readonly Content[] Electronics = { SalesL, Price2U, Price2L, TitleU, TitleL, RelDate2U, RelDate2L };
        public static readonly Content[] ForeignBooks = { SalesL, Price1U, PriceL, TitleU, TitleL, DateL };
        public static readonly Content[] HealthPersonalCare = { SalesL, Price2U, Price2L, TitleU, TitleL, RelDate2U, RelDate2L };
        public static readonly Content[] Hobbies = { SalesL, Price2U, Price2L, TitleU, TitleL, RelDate2U, RelDate2L, MinAgeU, MinAgeL };
        public static readonly Content[] Kitchen = { SalesL, Price2U, Price2L, TitleU, TitleL, RelDate2U, RelDate2L };
        public static readonly Content[] Music = { SalesL, Price1U, Price1L, TitleU, TitleL, RelDate1U, RelDate1L };
        public static readonly Content[] MusicTracks = { TitleU, TitleL };
        public static readonly Content[] Software = { SalesL, Price2U, Price2L, TitleU, TitleL, RelDate2U, RelDate2L };
        public static readonly Content[] SportingGoods = { SalesL, Price2U, Price2L, TitleU, TitleL, RelDate2U, RelDate2L };
        public static readonly Content[] Toys = { SalesL, Price2U, Price2L, TitleU, TitleL, RelDate2U, RelDate2L };
        public static readonly Content[] VHS = { SalesL, Price1U, Price1L, TitleU, TitleL, RelDate1U, RelDate1L };
        public static readonly Content[] Video = { SalesL, Price1U, Price1L, TitleU, TitleL, RelDate1U, RelDate1L };
        public static readonly Content[] VideoGames = { SalesL, Price2U, Price2L, TitleU, TitleL, RelDate2U, RelDate2L };
        public static readonly Content[] Watches = { SalesL, Price2U, Price2L, TitleU, TitleL };
    }

    public class Content
    {
        protected string myName, myValue;

        public virtual string Name
        {
            get { return myName; }
        }

        public virtual string Value
        {
            get { return myValue; }
        }

        public override string ToString()
        {
            return Value;
        }

        public Content(string name, string value)
        {
            myName = name;
            myValue = value;
        }
    }

    public class Store : Content
    {
        protected Content[] mySorts;

        public Content[] Sorts
        {
            get { return mySorts; }
        }

        public Store(string name, string value, Content[] sorts)
            : base(name, value)
        {
            mySorts = sorts;
        }
    }

    public class SearchIndex
    {
        public static readonly Store Blended =
            new Store("Blended", "ׂ",  SearchSort.Blended);
        public static readonly Store Books =
            new Store("Books", "{", SearchSort.Books);
        public static readonly Store Classical =
            new Store("Classical", "NVbNy", SearchSort.Classical);
        public static readonly Store DVD =
            new Store("DVD", "DVD", SearchSort.DVD);
        public static readonly Store Electronics =
            new Store("Electronics", "GNgjNX", SearchSort.Electronics);
        public static readonly Store ForeignBooks =
            new Store("ForeignBooks", "m",  SearchSort.ForeignBooks);
        public static readonly Store HealthPersonalCare =
            new Store("HealthPersonalCare", "wXr[eB[", SearchSort.HealthPersonalCare);
        public static readonly Store Hobbies =
            new Store("Hobbies", "zr[", SearchSort.Hobbies);
        public static readonly Store Kitchen =
            new Store("Kitchen", "z[Lb`",  SearchSort.Kitchen);
        public static readonly Store Music =
            new Store("Music", "y", SearchSort.Music);
        public static readonly Store MusicTracks =
            new Store("MusicTracks", "Ȗ",  SearchSort.MusicTracks);
        public static readonly Store Software =
            new Store("Software", "\tgEFA", SearchSort.Software);
        public static readonly Store SportingGoods =
            new Store("SportingGoods", "X|[cObY",  SearchSort.SportingGoods);
        public static readonly Store Toys = new Store("Toys", "",  SearchSort.Toys);
        public static readonly Store VHS = new Store("VHS", "VHS",  SearchSort.VHS);
        public static readonly Store Video = new Store("Video", "rfI", SearchSort.Video);
        public static readonly Store VideoGames = new Store("VideoGames", "Q[", SearchSort.VideoGames);
        public static readonly Store Watches = new Store("Watches", "v", SearchSort.Watches);
        public static readonly Store Baby = new Store("Baby", "xr[&}^jeB[", SearchSort.Baby);

        protected static readonly Store[] myAll = { Blended, Books, ForeignBooks, Electronics,
            Kitchen, Music, Classical, MusicTracks, DVD, VHS, Video, Software, VideoGames,
            Toys, Hobbies, SportingGoods, HealthPersonalCare, Watches, Baby };

        public static Store[] All
        {
            get { return myAll; }
        }
    }

}