using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using Amazon3.WebReference;

namespace Amazon3
{
    public partial class Form1 : Form
    {

        private string appId;

        public Form1()
        {
            InitializeComponent();
        }

        private void browseButton_Click(object sender, EventArgs e)
        {
            BrowseTreeNode selectedNode = treeView1.SelectedNode as BrowseTreeNode;
            if (selectedNode != null)
                Process.Start("http://www.amazon.co.jp/b/?node=" + selectedNode.NodeID);
        }

        private void SetInitialNode()
        {
            BrowseTreeNode[] all = {
                new BrowseTreeNode("{", "465392"),    
                new BrowseTreeNode("m", "52033011"),
                new BrowseTreeNode("GNgjNX", "3210981"),
                new BrowseTreeNode("z[&Lb`", "3828871"),
                new BrowseTreeNode("~[WbN", "561956"),
                new BrowseTreeNode("DVD", "561958"),
                new BrowseTreeNode("\tgEFA", "637392"),
                new BrowseTreeNode("Q[", "637394"),
                new BrowseTreeNode("&zr[", "13299531"),
                new BrowseTreeNode("X|[c&AEghA", "14304371"),
                new BrowseTreeNode("wX&r[eB[", "160384011"),
                new BrowseTreeNode("v", "324025011"),
                new BrowseTreeNode("xr[&}^jeB", "344845011")
            };

            treeView1.Nodes.Clear();
            treeView1.Nodes.AddRange(all);
        }

        private void settingButton_Click(object sender, EventArgs e)
        {
            using (Form2 idDialog = new Form2())
            {
                idDialog.ApplicationID = appId;

                if (idDialog.ShowDialog() == DialogResult.OK)
                {
                    appId = idDialog.ApplicationID;
                    Properties.Settings.Default.ApplicationId = appId;
                    Properties.Settings.Default.Save();
                }
            }
        }

        private void treeView1_BeforeExpand(object sender, TreeViewCancelEventArgs e)
        {
            try
            {
                BrowseTreeNode targetNode = e.Node as BrowseTreeNode;
                if (targetNode != null && !targetNode.IsGotten)
                {
                    BrowseNodeLookupResponse response =
                        BrowseNodeLookup(targetNode);

                    if (response != null)
                        DisplayResponse(targetNode, response);

                    targetNode.IsGotten = true;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
                e.Cancel = true;
            }
        }

        private BrowseNodeLookupResponse BrowseNodeLookup(BrowseTreeNode target)
        {
            BrowseNodeLookupResponse result = null;
            using (AWSECommerceService ecsService = new AWSECommerceService())
            {
                BrowseNodeLookup lookupOperation = new BrowseNodeLookup();
                lookupOperation.AWSAccessKeyId = appId;

                BrowseNodeLookupRequest lookupRequest = new BrowseNodeLookupRequest();

                lookupRequest.ResponseGroup = new string[] { "BrowseNodeInfo" };
                lookupRequest.BrowseNodeId = new string[] { target.NodeID };

                lookupOperation.Request = new BrowseNodeLookupRequest[] { lookupRequest };
                result = ecsService.BrowseNodeLookup(lookupOperation);
            }
            return result;
        }

        public void DisplayResponse(BrowseTreeNode parent, BrowseNodeLookupResponse response)
        {
            try
            {
                treeView1.BeginUpdate();

                ErrorCheck(response.OperationRequest.Errors);
                foreach (BrowseNodes nodes in response.BrowseNodes)
                {
                    ErrorCheck(nodes.Request.Errors);

                    parent.Nodes.Clear();
                    if (nodes.BrowseNode != null)
                        foreach (BrowseNode node in nodes.BrowseNode)
                        {
                            if (node.Children != null)
                                foreach (BrowseNode child in node.Children)
                                {
                                    BrowseTreeNode treeNode = new
                                        BrowseTreeNode(child.Name,
                                        child.BrowseNodeId);
                                    parent.Nodes.Add(treeNode);
                                }
                        }
                }
            }
            finally
            {
                treeView1.EndUpdate();
            }
        }

        private void ErrorCheck(ErrorsError[] errors)
        {
            if (errors != null)
            {
                string message = null;

                foreach (ErrorsError err in errors)
                    message += err.Message + "\r\n";

                throw new ApplicationException(message);
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            appId = Properties.Settings.Default.ApplicationId;
            SetInitialNode();
        }
    }

    public class BrowseTreeNode : TreeNode
    {
        protected string myNodeName = "";
        protected string myNodeID = "";
        protected bool isGotten;

        public string NodeName
        {
            get { return myNodeName; }
        }

        public string NodeID
        {
            get { return myNodeID; }
        }

        public bool IsGotten
        {
            set { isGotten = value; }
            get { return isGotten; }
        }

        public BrowseTreeNode(string name, string id)
        {
            myNodeName = name;
            myNodeID = id;
            IsGotten = false;

            Nodes.Add("");
            Text = NodeName + " (" + NodeID + ")";
        }
    }

}