using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using XmlRpc;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;
using System.IO;

namespace BlogPost
{
    public partial class Form1 : Form
    {
        public const string CocoUrl = "http://app.f.cocolog-nifty.com/t/api";
        public string userId, userPassword;
        private BlogInfo userBlogInfo;

        public Form1()
        {
            InitializeComponent();
        }

        private void imageButton_Click(object sender, EventArgs e)
        {

            if (userBlogInfo == null)
            {
                MessageBox.Show("[U[ݒsĂ", Text, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            using (Form3 imageDialog = new Form3())
            {
                if (imageDialog.ShowDialog() == DialogResult.OK)
                {
                    string imagePath = Path.GetFileNameWithoutExtension(imageDialog.TargetName) + ".jpg";
                    UploadImages(imageDialog.TargetImage, imagePath);
                }
            }
        }

        private void settingButton_Click(object sender, EventArgs e)
        {
            using (Form2 userDialog = new Form2())
            {
                userDialog.userId = userId;
                userDialog.userPassword = userPassword;
                userDialog.userBlog = userBlogInfo;

                if (userDialog.ShowDialog() == DialogResult.OK)
                {
                    userId = userDialog.userId;
                    userPassword = userDialog.userPassword;
                    userBlogInfo = userDialog.userBlog;
                }
            }
        }

        /// <summary>
        /// w肳ꂽC[WTlCƃIWiAbv[h
        /// </summary>
        /// <param name="targetImage"></param>
        /// <param name="targetName"></param>
        private void UploadImages(Image targetImage, string targetName)
        {
            if (targetImage == null)
                return;

            Image thumbNailImage = SmallImage(targetImage, 160);
            Image originalImage = null;

            int imageSize = Math.Max(targetImage.Width, targetImage.Height);

            if (imageSize > 200)
                originalImage = SmallImage(targetImage, 480);

            string thumbNailUrl = "", originalUrl = "";

            try
            {
                thumbNailUrl = UploadOneImage(thumbNailImage, "s_"
                    + targetName);

                if (originalImage != null)
                    originalUrl = UploadOneImage(originalImage, targetName);

                string linkTag = string.Format(@"<img src=""{0}"" border=""0"" width=""{1}"" height = ""{2}"" style=""float: right; margin: 0px 15px 5px 5px"">",
                    thumbNailUrl, thumbNailImage.Width.ToString(), thumbNailImage.Height.ToString());

                if (originalImage != null)
                {
                    linkTag = string.Format(@"<a href=""{0}"">{1}</a>",
                        originalUrl, linkTag);
                }
                contentsTextBox.Text += linkTag;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, Text, MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }
       }

        /// <summary>
        /// 1̃C[WAbv[h
        /// </summary>
        /// <param name="uploadImage"></param>
        private string UploadOneImage(Image uploadImage, string imageName)
        {
            byte[] upload = ImageToBytes(uploadImage, ImageFormat.Jpeg);
            XmlRpcNewMediaObject newMediaObject = 
                new XmlRpcNewMediaObject(CocoUrl, userBlogInfo.BlogId,
                userId, userPassword, imageName, upload);

            requestTextBox.Text = newMediaObject.RequestXmlText;
            string uploadUrl = newMediaObject.Request();

            return uploadUrl;
        }

        private Image SmallImage(Image original, int maxSize)
        {
            if (original == null)
                return null;

            int w = original.Size.Width;
            int h = original.Size.Height;
            double z = Math.Min((double)maxSize / Math.Max(w, h), 1);

            if (z < 1)
            {
                int nw = (int)Math.Floor(w * z);
                int nh = (int)Math.Floor(h * z);

                Image newImage = new Bitmap(nw, nh);
                using (Graphics g = Graphics.FromImage(newImage))
                {
                    g.InterpolationMode = InterpolationMode.HighQualityBilinear;
                    g.DrawImage(original, new Rectangle(0, 0, nw, nh));
                }
                return newImage;
            }
            else
            {
                return original;
            }
        }

        /// <summary>
        /// C[WoCgzɕϊ
        /// </summary>
        /// <param name="myImage"></param>
        /// <param name="format"></param>
        /// <returns></returns>
        private byte[] ImageToBytes(Image myImage, ImageFormat format)
        {
            MemoryStream myStream = new MemoryStream();
            myImage.Save(myStream, format);
            myStream.Close();

            return myStream.ToArray();
        }

        /// <summary>
        /// e{^NbNꂽ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void goButton_Click(object sender, EventArgs e)
        {
            if (userBlogInfo == null)
            {
                MessageBox.Show("[U[ݒsĂ", Text, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            // A[eBN쐬

            BlogArticle postArticle = new BlogArticle();

            postArticle.Title = titleTextBox.Text;
            postArticle.Description = contentsTextBox.Text;
            postArticle.DateCreated = dateTimePicker1.Value;
            postArticle.Mt_allow_comments = commentCheckBox.Checked ? 1 : 0;
            postArticle.Mt_allow_pings = trackbackCheckBox.Checked ? 1 : 0;
            postArticle.Mt_convert_breaks = "0";
            postArticle.Mt_excerpt = "";
            postArticle.Mt_text_more = "";
            postArticle.Mt_keywords = "";

            // XML-RPC\bhĂяo

            XmlRpcNewPost newPost = new XmlRpcNewPost(CocoUrl, userBlogInfo.BlogId, userId, userPassword, postArticle, publicCheckBox.Checked);
            requestTextBox.Text = newPost.RequestXmlText;
            
            newPost.Request();
        }

        private void tabControl1_SelectedIndexChanged(object sender, EventArgs e)
        {
            // vr[EBhEɃRec\
            if (tabControl1.SelectedIndex == 1)
            {
                webBrowser1.DocumentText = "<html><body>" + contentsTextBox.Text + "</body></html>";
            }
        }
     }
}