using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.IO;
using System.Net;
using System.Drawing.Imaging;

namespace XmlRpc
{
    /// <summary>
    /// metaWeblog.newPostĂяo
    /// </summary>
    public class XmlRpcNewPost : XmlRpcBase
    {
        private string userId, password, blogId, postid;
        private BlogArticle postArticle;
        private bool isPublished;

        public XmlRpcNewPost(string url, string blogid, string id,string pass, BlogArticle postBlog, bool publish)
            : base(url, "metaWeblog.newPost")
        {
            userId = id;
            blogId = blogid;
            password = pass;
            postArticle = postBlog;
            isPublished = publish;
        }

        public string Request()
        {
            CallRpcMethod();
            return postid;
        }

        public override void WriteParameter(XmlWriter myWriter)
        {
            XmlRpcParamWriter paramWriter = new XmlRpcParamWriter(myWriter);
            paramWriter.WriteParam(blogId);
            paramWriter.WriteParam(userId);
            paramWriter.WriteParam(password);

            myWriter.WriteStartElement("param");
            myWriter.WriteStartElement("struct");

            paramWriter.WriteStructMember("title", postArticle.Title);
            paramWriter.WriteStructMember("description", postArticle.Description);
            paramWriter.WriteStructMember("dateCreated", postArticle.DateCreated.ToUniversalTime());
            paramWriter.WriteStructMember("mt_allow_comments", postArticle.Mt_allow_comments);
            paramWriter.WriteStructMember("mt_allow_pings", postArticle.Mt_allow_pings);
            paramWriter.WriteStructMember("mt_convert_breaks", postArticle.Mt_convert_breaks);
            paramWriter.WriteStructMember("mt_text_more", postArticle.Mt_text_more);
            paramWriter.WriteStructMember("mt_excerpt", postArticle.Mt_excerpt);
            paramWriter.WriteStructMember("mt_keywords", postArticle.Mt_keywords);

            // struct
            myWriter.WriteEndElement();
            myWriter.WriteEndElement();

            // J邩ǂ
            paramWriter.WriteParam(isPublished);
        }

        public override void ParseReturnDoc(XmlDocument xmlDoc)
        {
            XmlNode structNode = xmlDoc.SelectSingleNode("//struct");

            if (structNode != null)
                postid = GetStructMember(structNode, "string", "postid");
        }
    }

    /// <summary>
    /// metaWeblog.newMediaObjectĂяo
    /// </summary>
    public class XmlRpcNewMediaObject : XmlRpcBase
    {
        private string userId, password, blogId, fileName, uploadUrl;
        private byte[] uploadData;

        public XmlRpcNewMediaObject(string url, string blogid, string id, string pass, string file, byte[] upload)
            : base(url, "metaWeblog.newMediaObject")
        {
            userId = id;
            blogId = blogid;
            password = pass;
            fileName = file;
            uploadData = upload;
        }

        public string Request()
        {
            CallRpcMethod();
            return uploadUrl;
        }

        public override void WriteParameter(XmlWriter myWriter)
        {
            XmlRpcParamWriter paramWriter = new XmlRpcParamWriter(myWriter);
            paramWriter.WriteParam(blogId);
            paramWriter.WriteParam(userId);
            paramWriter.WriteParam(password);

            myWriter.WriteStartElement("param");
            myWriter.WriteStartElement("value");
            myWriter.WriteStartElement("struct");

            paramWriter.WriteStructMember("bits", uploadData);
            paramWriter.WriteStructMember("name", fileName);

            myWriter.WriteEndElement();
            myWriter.WriteEndElement();
            myWriter.WriteEndElement();
        }

        public override void ParseReturnDoc(XmlDocument xmlDoc)
        {
            XmlNode structNode = xmlDoc.SelectSingleNode("//struct");

            if (structNode != null)
                uploadUrl = GetStructMember(structNode, "string", "url");
        }

    }

    /// <summary>
    /// blogger.getUsersBlogsĂяo
    /// </summary>
    public class XmlRpcGetUsersBlogs : XmlRpcBase
    {
        private string userId, password;
        private List<BlogInfo> blogInfoList;

        public XmlRpcGetUsersBlogs(string url, string id, string pass)
            : base(url, "blogger.getUsersBlogs")
        {
            userId = id;
            password = pass;
        }

        public List<BlogInfo> Request()
        {
            CallRpcMethod();
            return blogInfoList;
        }

        public override void WriteParameter(XmlWriter myWriter)
        {
            XmlRpcParamWriter paramWriter =
                new XmlRpcParamWriter(myWriter);
            paramWriter.WriteParam("");
            paramWriter.WriteParam(userId);
            paramWriter.WriteParam(password);
        }

        public override void ParseReturnDoc(XmlDocument xmlDoc)
        {
            XmlNodeList structNodes = xmlDoc.SelectNodes("//struct");
            blogInfoList = new List<BlogInfo>();

            foreach (XmlNode currentNode in structNodes)
            {
                BlogInfo myInfo = new BlogInfo();
                myInfo.Url = GetStructMember(currentNode, "string", "url");
                myInfo.BlogId = GetStructMember(currentNode, "string",
                    "blogid");
                myInfo.BlogName = GetStructMember(currentNode, "string",
                    "blogName");
                blogInfoList.Add(myInfo);
            }
        }
    }

    /// <summary>
    /// metaWeblog.getRecentPostsĂяo
    /// </summary>
    public class XmlRpcGetRecentPosts : XmlRpcBase
    {
        private string userId, password, blogId;
        private int postCount;
        private List<BlogArticle> blogArticleList;

        public XmlRpcGetRecentPosts(string url, string blogid, string id, string pass, int number)
            : base(url, "metaWeblog.getRecentPosts")
        {
            userId = id;
            password = pass;
            blogId = blogid;
            postCount = number;
        }

        public List<BlogArticle> Request()
        {
            CallRpcMethod();
            return blogArticleList;
        }

        public override void WriteParameter(XmlWriter myWriter)
        {
            XmlRpcParamWriter paramWriter = new XmlRpcParamWriter(myWriter);
            paramWriter.WriteParam(blogId);
            paramWriter.WriteParam(userId);
            paramWriter.WriteParam(password);
            paramWriter.WriteParam(postCount);
        }

        public override void ParseReturnDoc(XmlDocument xmlDoc)
        {
            XmlNodeList structNodes = xmlDoc.SelectNodes("//struct");
            blogArticleList = new List<BlogArticle>();

            foreach (XmlNode currentNode in structNodes)
            {
                BlogArticle myArticle = new BlogArticle();
                myArticle.Link = GetStructMember(currentNode, "string", "link");
                myArticle.PermaLink = GetStructMember(currentNode, "string", "permaLink");
                myArticle.UserId = GetStructMember(currentNode, "string", "userid");
                myArticle.PostId = GetStructMember(currentNode, "string", "postid");
                myArticle.Title = GetStructMember(currentNode, "string", "title");
                myArticle.DateCreated = GetDateTime(GetStructMember(currentNode, "dateTime.iso8601", "dateCreated"));
                myArticle.Description = GetStructMember(currentNode, "string", "description");

                blogArticleList.Add(myArticle);
            }
        }
    }

    /// <summary>
    /// [UpłBlog̈ꗗ
    /// </summary>
    public class BlogInfo
    {
        private string url, blogId, blogName;

        public string Url
        {
            get { return url; }
            set { url = value; }
        }

        public string BlogId
        {
            get { return blogId; }
            set { blogId = value; }
        }

        public string BlogName
        {
            get { return blogName; }
            set { blogName = value; }
        }

        public override string ToString()
        {
            return BlogName;
        }
    }

    /// <summary>
    /// XML-RPC̋ʏ
    /// </summary>
    public abstract class XmlRpcBase
    {
        private string baseUrl, methodName;

        private string responseXmlText;
        private int faultCode;
        private string faultMessage;

        protected XmlRpcBase(string url, string name)
        {
            baseUrl = url;
            methodName = name;
        }

        /// <summary>
        /// RXgN^Ŏw肳ꂽp[^烊NGXgXML擾
        /// </summary>
        /// <returns></returns>
        /// 
        public string RequestXmlText
        {
            get
            {
                MemoryStream myStream = new MemoryStream();
                SendXml(myStream);
                myStream.Close();

                return Encoding.UTF8.GetString(myStream.ToArray());
            }
        }

        protected void CallRpcMethod()
        {
            HttpWebRequest myRequest =
                (HttpWebRequest)WebRequest.Create(baseUrl);

            myRequest.Method = "POST";
            myRequest.ContentType = "text/xml";
            myRequest.UserAgent = "blogclient 1.0.0";

            Stream requestStream = myRequest.GetRequestStream();

            SendXml(requestStream);
            requestStream.Close();

            WebResponse myResponse = null;
            XmlDocument returnDoc = new XmlDocument();

            try
            {
                // \bhĂяo̎s
                myResponse = myRequest.GetResponse();

                // ߂lXmlDocumntɑ
                Stream responseStream = myResponse.GetResponseStream();
                returnDoc.Load(responseStream);
            }
            catch
            {
                throw new ApplicationException("XML-RPC̎sɎs܂");
            }
            finally
            {
                if (myResponse != null)
                    myResponse.Close();
            }

            responseXmlText = returnDoc.OuterXml;

            if (returnDoc.SelectNodes("/methodResponse/fault").Count > 0)
            {
                XmlNode faultStruct = returnDoc.SelectSingleNode("//struct");
                faultCode = int.Parse(GetStructMember(faultStruct,
                    "int", "faultCode"));
                faultMessage = GetStructMember(faultStruct, "string",
                    "faultString");
                throw new ApplicationException("XML-RPC̎sɎs܂");
            }
            else
            {
                ParseReturnDoc(returnDoc);
            }
        }

        private void SendXml(Stream sendStream)
        {
            XmlWriter myWriter = XmlWriter.Create(sendStream);

            myWriter.WriteStartElement("methodCall");
            myWriter.WriteElementString("methodName", methodName);
            myWriter.WriteStartElement("params");

            WriteParameter(myWriter);

            myWriter.WriteEndElement();
            myWriter.WriteEndElement();
            myWriter.Close();
        }

        protected string GetStructMember(XmlNode structNode, string membertype, string membername)
        {
            XmlNode valueNode =
                structNode.SelectSingleNode("member/value[../name/text() = \"" +
                membername + "\"]");

            if (valueNode != null)
            {
                foreach (XmlNode childNode in valueNode.ChildNodes)
                {
                    if (childNode.NodeType == XmlNodeType.Element)
                    {
                        if (childNode.Name == membertype)
                            return childNode.InnerText;
                        if (childNode.Name == "i4" && membertype == "int")
                            return childNode.InnerText;
                    }
                    else if (childNode.NodeType == XmlNodeType.Text && membertype == "string")
                        return valueNode.InnerText;
                }
            }
            return "";
        }

        public abstract void ParseReturnDoc(XmlDocument xmlDoc);

        public abstract void WriteParameter(XmlWriter myWriter);

        public DateTime GetDateTime(string s)
        {
            string[] dateformat = new string[4] { "yyyy-MM-ddTHH:mm:ss.FFFzzz", "yyyy-MM-ddTHH:mm:ss.FFFZ", "yyyy-MM-dd", "ddd, dd MMM yyyy HH:mm:ss zzz" };
            DateTime d = DateTime.Parse(s);
            return d;
        }

        public string ResponseXmlText
        {
            get { return responseXmlText; }
        }

        public int FaultCode
        {
            get { return faultCode; }
        }

        public string FaultMessage
        {
            get { return faultMessage; }
        }
    }

    /// <summary>
    /// XML-RPC̃p[^o͂NX
    /// </summary>
    public class XmlRpcParamWriter
    {
        private XmlWriter myWriter;

        public XmlRpcParamWriter(XmlWriter writer)
        {
            myWriter = writer;
        }

        public void WriteParam(object x)
        {
            myWriter.WriteStartElement("param");
            WriteValue(x);
            myWriter.WriteEndElement();
        }

        public void WriteStructMember(string memberName, object x)
        {
            myWriter.WriteStartElement("member");
            myWriter.WriteElementString("name", memberName);
            WriteValue(x);
            myWriter.WriteEndElement();
        }

        public void WriteValue(object x)
        {
            string parameterType, parameterText;

            if (x is int)
            {
                parameterType = "int";
                parameterText = x.ToString();
            }
            else if (x is string)
            {
                parameterType = "string";
                parameterText = x.ToString();
            }
            else if (x is bool)
            {
                parameterType = "boolean";
                parameterText = (bool)x ? "1" : "0";
            }
            else if (x is byte[])
            {
                parameterType = "base64";
                parameterText = Convert.ToBase64String((byte[])x);
            }
            else if (x is DateTime)
            {
                DateTime myDateTime = (DateTime)x;
                parameterType = "dateTime.iso8601";
                if (myDateTime.Kind == DateTimeKind.Utc)
                    parameterText = myDateTime.ToString("yyyy-MM-ddTHH:mm:ssZ");
                else
                    parameterText = myDateTime.ToString("yyyy-MM-ddTHH:mm:sszzz");
            }
            else
            {
                throw new ApplicationException("T|[gĂȂ^w肳܂");
            }

            myWriter.WriteStartElement("value");
            myWriter.WriteElementString(parameterType, parameterText);
            myWriter.WriteEndElement();
        }
    }
    /// <summary>
    /// Xgr[\pBlogArtice
    /// </summary>
    public class BlogArticleItem : ListViewItem
    {
        private BlogArticle blogItem;

        public BlogArticleItem()
        {
            SubItems.Clear();
            SubItems.AddRange(new string[2] { "", "" });
        }

        public BlogArticle Item
        {
            get { return blogItem; }
            set
            {
                blogItem = value;
                Text = blogItem.DateCreated.ToString();
                SubItems[1].Text = value.Title;
                SubItems[2].Text = value.Mt_keywords;
            }
        }
    }

    /// <summary>
    /// Blog̋L\
    /// 擾pep
    /// </summary>
    public class BlogArticle
    {
        private string link, permaLink, userid, 
            description, mt_convert_breaks, postid, mt_excerpt, mt_keywords,
            title, mt_text_more;

        private int mt_allow_pings, mt_allow_comments;
        private DateTime dateCreated;

        public string Link
        {
            get { return link; }
            set { link = value; }
        }

        public string PermaLink
        {
            get { return permaLink; }
            set { permaLink = value; }
        }

        public string UserId
        {
            get { return userid; }
            set { userid = value; }
        }

        public int Mt_allow_pings
        {
            get { return mt_allow_pings; }
            set { mt_allow_pings = value; }
        }

        public int Mt_allow_comments
        {
            get { return mt_allow_comments; }
            set { mt_allow_comments = value; }
        }

        public string Description
        {
            get { return description; }
            set { description = value; }
        }

        public string Mt_convert_breaks
        {
            get { return mt_convert_breaks; }
            set { mt_convert_breaks = value; }
        }

        public string PostId
        {
            get { return postid; }
            set { postid = value; }
        }

        public string Mt_excerpt
        {
            get { return mt_excerpt; }
            set { mt_excerpt = value; }
        }

        public string Mt_keywords
        {
            get { return mt_keywords; }
            set { mt_keywords = value; }
        }

        public string Title
        {
            get { return title; }
            set { title = value; }
        }

        public string Mt_text_more
        {
            get { return mt_text_more; }
            set { mt_text_more = value; }
        }

        public DateTime DateCreated
        {
            get { return dateCreated; }
            set { dateCreated = value; }
        }
    }
}