using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Google.GData.Calendar;
using Google.GData.Client;
using Google.GData.Extensions;

namespace EventList1
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void goButton_Click(object sender, EventArgs e)
        {

            CalendarService gCalService = new CalendarService("PHL-GCalClient-1.0");

            EventQuery myQuery = new EventQuery(urlTextBox.Text);

            myQuery.ExtraParameters = "orderby=starttime";
            myQuery.SingleEvents = true;
            myQuery.SortOrder = CalendarSortOrder.ascending;
            myQuery.StartTime = DateTime.Now.Date;
            myQuery.EndTime = myQuery.StartTime.AddMonths(3);
            myQuery.NumberToRetrieve = (int)numericUpDown1.Value;

            if (emailTextBox.Text.Length > 0)
            {
                gCalService.setUserCredentials(emailTextBox.Text, passwordMaskedTextBox.Text);
            }

            EventFeed myFeed = null;

            int tryCount = 2;

            do
            {
                try
                {
                    myFeed = gCalService.Query(myQuery);
                    tryCount = 0;
                }
                catch (Exception ex)
                {
                    if (--tryCount < 1 || !(ex is GDataRequestException))
                    {
                        // GData Client LibrarỹoȎΉ
                        MessageBox.Show(ex.Message, this.Text);
                        return;
                    }
                }
            } while (tryCount > 0);

            if (myFeed != null)
            {
                listView1.Items.Clear();

                foreach (EventEntry currentEvent in myFeed.Entries)
                {
                    EventEntryItem eventItem = new EventEntryItem();
                    eventItem.Item = currentEvent;
                    listView1.Items.Add(eventItem);
                }
            }
        }
    }

    public class EventEntryItem : ListViewItem
    {
        EventEntry item = null;
        
        public EventEntryItem()
        {
            SubItems.Clear();
        }

        public EventEntry Item
        {
            get { return item; }
            set
            {
                SubItems.Clear();
                item = value;

                string when = "--";

                if (item.Times.Count > 0)
                {
                    When currentWhen = item.Times[0];

                    if (currentWhen.AllDay)
                    {
                        DateTime fillEndTime =
                            currentWhen.EndTime.Date.AddDays(-1);
                        if (currentWhen.StartTime.Date == fillEndTime.Date)
                            when = string.Format("{0:yyyy/MM/dd}", currentWhen.StartTime);
                        else
                            when = string.Format(
                                "{0:yyyy/MM/dd}`{1:yyyy/MM/dd}",
                                currentWhen.StartTime, fillEndTime);
                    }
                    else
                    {
                        if (currentWhen.StartTime == currentWhen.EndTime)
                            when = string.Format("{0:yyyy/MM/dd HH:mm}",
                                currentWhen.StartTime);
                        else
                            when = string.Format(
                                "{0:yyyy/MM/dd HH:mm}`{1:HH:mm}",
                                currentWhen.StartTime, currentWhen.EndTime);
                    }
                }

                string visibility = "--";

                if (item.EventVisibility != null)
                {
                    if (item.EventVisibility.equals(EventEntry.Visibility.CONFIDENTIAL))
                        visibility = "閧";

                    if (item.EventVisibility.equals(EventEntry.Visibility.PRIVATE))
                        visibility = "J";

                    if (item.EventVisibility.equals(EventEntry.Visibility.PUBLIC))
                        visibility = "J";

                    if (item.EventVisibility.equals(EventEntry.Visibility.DEFAULT))
                        visibility = "ftHg";
                }

                Text = item.Title == null ? "no-title" : item.Title.Text;
                SubItems.Add(item.Locations.Count > 0 ? item.Locations[0].ValueString : "--");
                SubItems.Add(when);
                SubItems.Add(visibility);
            }
        }
    }
}