using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Gcalendar;
using System.Web;

namespace EventList3
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void goButton_Click(object sender, EventArgs e)
        {
            DateTime startTime = DateTime.Today;
            DateTime endTime = startTime.AddMonths(3);

            string startTimeDesc =HttpUtility.UrlEncode(startTime.ToString("yyyy-MM-ddTHH:mm:sszzz"));
            string endTimeDesc =HttpUtility.UrlEncode(endTime.ToString("yyyy-MM-ddTHH:mm:sszzz"));

            string feedUrl = string.Format("{0}?orderby=starttime&singleevents=true&sortorder=ascending&start-min={1}&start-max={2}&max-results={3}",
                urlTextBox.Text, startTimeDesc, endTimeDesc,numericUpDown1.Value);

            GoogleAuth myAuth = null;

            if (emailTextBox.Text.Length > 0)
                myAuth = new GoogleAuth(emailTextBox.Text, passwordTextBox.Text, "cl");

            EventFeed myFeed = new EventFeed(feedUrl, myAuth);
            List<EventEntry> eventList = null;
            
            try
            {
                eventList = myFeed.Query();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, this.Text);
                return;
            }

            feedTextBox.Text = myFeed.FeedText;

            if (eventList != null)
            {
                listView1.Items.Clear();

                foreach (EventEntry currentEvent in eventList)
                {
                    EventEntryItem eventItem = new EventEntryItem();
                    eventItem.Item = currentEvent;
                    listView1.Items.Add(eventItem);
                }
            }
        }

        private void calButton_Click(object sender, EventArgs e)
        {
            calendarComboBox.Items.Clear();

            string feedUrl = "http://www.google.com/calendar/feeds/" + HttpUtility.UrlEncode(emailTextBox.Text);

            GoogleAuth myAuth = new GoogleAuth(emailTextBox.Text, passwordTextBox.Text, "cl");

            CalendarFeed myFeed = new CalendarFeed(feedUrl, myAuth);
            List<CalendarEntry> calendarList = null;

            try
            {
                calendarList = myFeed.Query();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, this.Text);
                return;
            }

            feedTextBox.Text = myFeed.FeedText;

            foreach (CalendarEntry currentEntry in calendarList)
            {
                CalendarItem currentCalendar = new CalendarItem(currentEntry);
                calendarComboBox.Items.Add(currentCalendar);
            }
        }

        private void calendarComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (calendarComboBox.SelectedItem != null)
            {
                urlTextBox.Text = ((CalendarItem)calendarComboBox.SelectedItem).FeedUrl;
            }
        }
    }

    class CalendarItem
    {
        private string title, feedUrl;

        public CalendarItem(CalendarEntry entry)
        {
            title = entry.Title;
            feedUrl = entry.FeedUrl;
        }

        public string FeedUrl
        {
            get { return feedUrl; }
        }

        public string Title
        {
            get { return title; }
        }

        public override string ToString()
        {
            return title;
        }
    }
    
    class EventEntryItem : ListViewItem
    {
        EventEntry item;
        
        public EventEntryItem()
        {
            item = null;
            SubItems.Clear();
        }

        public EventEntry Item
        {
            get { return item; }
            set
            {
                item = value;

                string when = "--";

                if (item.StartTime.HasValue)
                {
                    if (item.IsAllDay)
                    {
                        DateTime fillEndTime = ((DateTime)item.EndTime).AddDays(-1);
                        if (((DateTime)item.StartTime).Date == fillEndTime.Date)
                            when = string.Format("{0:yyyy/MM/dd}", item.StartTime);
                        else
                            when = string.Format("{0:yyyy/MM/dd}`{1:yyyy/MM/dd}", item.StartTime, fillEndTime);
                    }
                    else
                    {
                        if (item.StartTime == item.EndTime)
                            when = string.Format("{0:yyyy/MM/dd HH:mm}", item.StartTime);
                        else
                            when = string.Format("{0:yyyy/MM/dd HH:mm}`{1:HH:mm}", item.StartTime, item.EndTime);
                    }
                }

                string visibility = "--";

                if (item.Visibility == EventVisibility.Confidential)
                    visibility = "閧";
                
                if (item.Visibility == EventVisibility.Private)
                    visibility = "J";

                if (item.Visibility == EventVisibility.Public)
                    visibility = "J";

                if (item.Visibility == EventVisibility.Default)
                    visibility = "ftHg";

                Text = item.Title;
                
                SubItems.Add(item.Where);
                SubItems.Add(when);
                SubItems.Add(visibility);
            }
        }
    }
}