using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Google.GData.Calendar;
using Google.GData.Client;
using Google.GData.Extensions;
using System.Xml;
using System.IO;
using System.Web;

namespace EventPost
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        CalendarService gCalService;

        private void calButton_Click(object sender, EventArgs e)
        {
            calendarComboBox.Items.Clear();

            FeedQuery myQuery = new FeedQuery("http://www.google.com/calendar/feeds/" +
                HttpUtility.UrlEncode(emailTextBox.Text));
            gCalService.setUserCredentials(emailTextBox.Text, passwordMaskedTextBox.Text);

            AtomFeed myFeed = null;

            try
            {
                myFeed = gCalService.Query(myQuery);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, this.Text);
                return;
            }

            foreach (AtomEntry currentEntry in myFeed.Entries)
            {
                CalendarItem currentCalendar = new CalendarItem(currentEntry);
                if (currentCalendar.IsWritable)
                    calendarComboBox.Items.Add(currentCalendar);
            }
        }

        private bool ValidateEntry(EventEntry entry)
        {
            string message = null;

            if (entry.Title.Text.Length < 1)
                message = "^Cg͂Ă\r\n";

            if (entry.Times[0].StartTime > entry.Times[0].EndTime)
                message += "IJnɂȂĂ܂\r\n";

            if (message != null)
                MessageBox.Show(message, this.Text);

            return (message == null);
        }

        private EventEntry GetEntry()
        {
            EventEntry newEntry = new EventEntry();
            newEntry.Title.Text = titleTextBox.Text;

            When eventWhen = new When();
            eventWhen.AllDay = allDayCheckBox.Checked;
            if (eventWhen.AllDay)
            {
                eventWhen.StartTime = startDateTimePicker1.Value;
                eventWhen.EndTime = endDateTimePicker1.Value.AddDays(1);
            }
            else
            {
                eventWhen.StartTime = MixDateTime(startDateTimePicker1,
                    startDateTimePicker2);
                eventWhen.EndTime = MixDateTime(endDateTimePicker1,
                    endDateTimePicker2);
            }

            newEntry.Times.Add(eventWhen);

            if (opaqueCheckBox.Checked)
                newEntry.EventTransparency =
                    EventEntry.Transparency.OPAQUE;
            else
                newEntry.EventTransparency =
                    EventEntry.Transparency.TRANSPARENT;

            return newEntry;
         }

        private DateTime MixDateTime(DateTimePicker dt1, DateTimePicker dt2)
        {
            DateTime day1 = dt1.Value.Date;
            DateTime day2 = day1.AddTicks(dt2.Value.TimeOfDay.Ticks);

            return day2;
        }

        private void goButton_Click(object sender, EventArgs e)
        {
            EventEntry newEntry = GetEntry();
            if (ValidateEntry(newEntry))
            {
                string newUrl = urlTextBox.Text;
                EventQuery myQuery = new EventQuery();
                myQuery.Uri = new Uri(urlTextBox.Text);

                gCalService.setUserCredentials(emailTextBox.Text, passwordMaskedTextBox.Text);

                try
                {
                    EventFeed calFeed = gCalService.Query(myQuery);
                    EventEntry newEntry2 = (EventEntry)(calFeed.Insert(newEntry));
                    newEntry2.Title.Text = "ύX܂";
                    newEntry2.Update();
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void calendarComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (calendarComboBox.SelectedItem != null)
            {
                urlTextBox.Text = ((CalendarItem)calendarComboBox.SelectedItem).FeedUrl;
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            gCalService = new CalendarService("PHL-GcalClient-1.0");
        }

        private void allDayCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            startDateTimePicker2.Visible = !allDayCheckBox.Checked;
            endDateTimePicker2.Visible = !allDayCheckBox.Checked;
        }
    }

    class CalendarItem
    {
        private string title, feedUrl;
        private bool isWritable;

        public CalendarItem(AtomEntry entry)
        {
            foreach (Object x in entry.ExtensionElements)
            {
                if (x is XmlNode)
                {
                    XmlNode myNode = (XmlNode)x;
                    if (myNode.NodeType == XmlNodeType.Element && 
                        myNode.NamespaceURI ==
                            "http://schemas.google.com/gCal/2005" &&
                            myNode.LocalName == "accesslevel")
                    {
                        XmlAttribute accessAttr;
                        string accessLevel = (accessAttr = myNode.Attributes["value"]) ==
                            null ? "" : accessAttr.Value;

                        isWritable = (accessLevel == "owner" || accessLevel == "contributor");
                    }
                }
            }

            title = entry.Title == null ? "no-title" : entry.Title.Text;

            foreach (AtomLink currentLink in entry.Links)
            {
                if (currentLink.Rel == "alternate")
                    feedUrl = currentLink.AbsoluteUri;
            }
        }

        public string FeedUrl
        {
            get { return feedUrl; }
        }

        public string Title
        {
            get { return title; }
        }

        public bool IsWritable
        {
            get { return isWritable; }
        }

        public override string ToString()
        {
            return title;
        }
    }
}