using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Text.RegularExpressions;
using System.IO;

namespace HttpDownload4
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void goButton_Click(object sender, EventArgs e)
        {
            WebRequest myRequest =
                WebRequest.Create(urlTextBox.Text);

            HttpWebResponse myResponse = null;

            try
            {
                myResponse =
                    (HttpWebResponse)myRequest.GetResponse();
            }
            catch (WebException ex)
            {
                if (ex.Status ==
                    WebExceptionStatus.ProtocolError)
                {
                    myResponse = (HttpWebResponse)ex.Response;
                }
                else
                {
                    MessageBox.Show(ex.Message, Text,
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
                }
            }

            if (myResponse != null)
            {
                int code = (int)myResponse.StatusCode;
                string description =
                    myResponse.StatusDescription;
                
                Encoding contentEncoding;
                Stream responseStream =
                    myResponse.GetResponseStream();
                byte[] contents = StreamToArray(responseStream);
                myResponse.Close();

                if (sjisRadioButton.Checked)
                    contentEncoding =
                        Encoding.GetEncoding("shift_jis");
                else if (eucRadioButton.Checked)
                    contentEncoding =
                        Encoding.GetEncoding("euc-jp");
                else if (utf8RadioButton.Checked)
                    contentEncoding = Encoding.GetEncoding("utf-8");
                else
                    contentEncoding = DetectEncoding(contents);

                htmlTextBox.Text = "status : " + code + " " +
                    description + "\r\n-----\r\n\r\n" +
                    contentEncoding.GetString(contents);
                encodeLabel.Text = contentEncoding.WebName;
            }
        }

        private Encoding DetectEncoding(byte[] contents)
        {
            string s = Encoding.ASCII.GetString(contents);

            Match myMatch = Regex.Match(s, 
                @"<meta\s+[^>]*charset\s*=\s*([-_\w]+)", 
                RegexOptions.IgnoreCase);
            string e = myMatch.Success ? myMatch.Groups[1].Value : 
                "shift_jis";
            return Encoding.GetEncoding(e);
        }

        private byte[] StreamToArray(Stream contents)
        {
            int offset = 0;
            const int ReadSize = 8192;

            byte[] currentBytes = new byte[ReadSize];
            
            int f = 0;
            while ((f = contents.Read(currentBytes, offset, ReadSize)) > 0)
            {
                byte[] newBytes = new byte[offset + f + ReadSize];
                Array.Copy(currentBytes, newBytes, offset + f);
                offset += f;
                currentBytes = newBytes;
            }

            byte[] returnBytes = new byte[offset];
            Array.Copy(currentBytes, returnBytes, offset);
            return returnBytes;
        }
    }
}