using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.Net;
using System.IO;
using System.Globalization;

namespace LWeather
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void goButton_Click(object sender, EventArgs e)
        {
            if (comboBox3.SelectedIndex != -1)
            {
                try
                {
                    Area myArea = (Area)comboBox3.SelectedItem;
                    WeatherRequest myWeather = new WeatherRequest(myArea.ID, "today");
                    WeatherInfo myInfo = myWeather.GetWeatherInfo();

                    cityLabel.Text = myInfo.Pref + "  " + myInfo.City + "̓VC";
                    DisplayWeather(myInfo, day1Label, telop1Label,
                        max1Label, min1Label, pictureBox1);

                    myWeather = new WeatherRequest(myArea.ID, "tomorrow");
                    myInfo = myWeather.GetWeatherInfo();
                    DisplayWeather(myInfo, day2Label, telop2Label,
                        max2Label, min2Label, pictureBox2);

                    myWeather = new WeatherRequest(myArea.ID, "dayaftertomorrow");
                    myInfo = myWeather.GetWeatherInfo();
                    DisplayWeather(myInfo, day3Label, telop3Label,
                        max3Label, min3Label, pictureBox3);
                    publicLabel.Text =
                        myInfo.PublicTime.ToString("MMddHH\");
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void DisplayWeather(WeatherInfo w, Label daylabel,
            Label telop, Label max, Label min, PictureBox p)
        {
            daylabel.Text = w.ForecastDate.ToString("MMdd(ddd)");
            telop.Text = w.Telop;
            max.Text = (w.TempMaxCelsius == null ? "--" :
                w.TempMaxCelsius.ToString()) + "";
            min.Text = (w.TempMinCelsius == null ? "--" :
                w.TempMinCelsius.ToString()) + "";
            p.Image = w.WeatherIcon; 
        }

        private void AddDropDown(ComboBox c, Area a)
        {
            c.Items.Clear();
            foreach(Area x in a.Childs)
                c.Items.Add(x);
        }

        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            AddDropDown(comboBox2, (Area)comboBox1.SelectedItem);
            comboBox2.SelectedIndex = 0;
        }

        private void comboBox2_SelectedIndexChanged(object sender, EventArgs e)
        {
            AddDropDown(comboBox3, (Area)comboBox2.SelectedItem);
            comboBox3.SelectedIndex = 0;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            AreaRequest myRequest = new AreaRequest();

            try
            {
                Area rootArea = myRequest.GetRootArea();

                AddDropDown(comboBox1, rootArea);
                comboBox1.SelectedIndex = 0;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, Text);
            }
        }
    }

    public class Area
    {
        private string title, id;
        private List<Area> childs = new List<Area>();

        public string Title
        {
            get { return title; }
            set { title = value; }
        }

        public string ID
        {
            get { return id; }
            set { id = value; }
        }

        public List<Area> Childs
        {
            get { return childs; }
        }

        public Area(string myTitle, string myId)
        {
            Title = myTitle;
            ID = myId;
        }

        public override string ToString()
        {
            return Title;
        }
    }

    public class AreaRequest
    {
        public Area GetRootArea()
        {
            const string areaUrl =
                "http://weather.livedoor.com/forecast/rss/forecastmap.xml";

            WebRequest areaWebRequset = WebRequest.Create(areaUrl);
            WebResponse areaResponse = null;
            XmlDocument areaDoc = new XmlDocument();

            try
            {
                areaResponse = areaWebRequset.GetResponse();
                Stream areaStream = areaResponse.GetResponseStream();
                areaDoc.Load(areaStream);
            }
            catch 
            {
                throw new ApplicationException("nXML擾ł܂");
            }
            finally
            {
                if (areaResponse != null)
                    areaResponse.Close();
            }

            XmlNodeList rootNodes = areaDoc.GetElementsByTagName("source",
                "http://weather.livedoor.com/ns/rss/2.0");
            Area rootArea = new Area("S", null);

            if (rootNodes == null)
                throw new ApplicationException("nXML܂");
            else
                GetAreas(rootNodes[0], rootArea);

            return rootArea;

        }

        private void GetAreas(XmlNode currentNode, Area currentArea)
        {
            foreach (XmlNode child in currentNode.ChildNodes)
            {
                if (child.NodeType == XmlNodeType.Element &&
                    (child.Name == "area" || child.Name == "pref" ||
                    child.Name == "city"))
                {
                    XmlNode idNode = child.Attributes["id"];
                    string cityId = (idNode == null) ?
                        null : idNode.Value;

                    XmlNode titleNode = child.Attributes["title"];
                    string title = (titleNode == null) ?
                        "" : titleNode.Value;

                    Area newArea = new Area(title, cityId);
                    GetAreas(child, newArea);
                    currentArea.Childs.Add(newArea);
                }
            }
        }
    }

    public class WeatherRequest
    {
        private string cityID;
        private string forecastDay;

        public WeatherRequest(string id, string day)
        {
            cityID = id;
            forecastDay = day;
        }

        public WeatherInfo GetWeatherInfo()
        {
            const string weatherUrl =
                "http://weather.livedoor.com/forecast/webservice/rest/v1?city={0}&day={1}";
            string RequestUrl =
                string.Format(weatherUrl, cityID, forecastDay);

            WebResponse weatherResponse = null;
            XmlDocument weatherDoc = new XmlDocument();

            try
            {
                weatherResponse =
                    WebRequest.Create(RequestUrl).GetResponse();
                Stream weatherStream =
                    weatherResponse.GetResponseStream();
                weatherDoc.Load(weatherStream);
            }
            catch
            {
                throw new ApplicationException("VC\񂪎擾ł܂");
            }
            finally
            {
                if (weatherResponse != null)
                    weatherResponse.Close();
            }
            return MakeWeatherInfo(weatherDoc);
        }

        private WeatherInfo MakeWeatherInfo(XmlDocument doc)
        {
            WeatherInfo myWeather = new WeatherInfo();
            
            XmlNode rootElement = doc.DocumentElement;
            myWeather.Area =
                rootElement["location"].Attributes["area"].Value;
            myWeather.Pref =
                rootElement["location"].Attributes["pref"].Value;
            myWeather.City =
                rootElement["location"].Attributes["city"].Value;
            myWeather.Title = rootElement["title"].InnerText;
            myWeather.ForecastDay = rootElement["forecastday"].InnerText;
            myWeather.Day = rootElement["day"].InnerText;
            myWeather.ForecastDate = GetWeatherDate(rootElement["forecastdate"].InnerText);
            myWeather.PublicTime = GetWeatherDate(rootElement["publictime"].InnerText);
            myWeather.Telop = rootElement["telop"].InnerText;
            myWeather.ImageUrl = rootElement["image"]["url"].InnerText;
            myWeather.ImageWidth = int.Parse(rootElement["image"]["width"].InnerText);
            myWeather.ImageHeight = int.Parse(rootElement["image"]["height"].InnerText);
            myWeather.TempMaxCelsius = GetDoubleOrNull(
                rootElement["temperature"]["max"]["celsius"].InnerText);
            myWeather.TempMinCelsius = GetDoubleOrNull(
                rootElement["temperature"]["min"]["celsius"].InnerText);

            return myWeather;
        }

        private DateTime GetWeatherDate(string s)
        {
            return DateTime.ParseExact(s, "ddd, dd MMM yyyy HH:mm:ss zzz",
                CultureInfo.InvariantCulture);
        }

        private double? GetDoubleOrNull(string s)
        {
            double d;
            if (double.TryParse(s, out d))
                return d;
            else
                return null;
        }
    }

    public class WeatherInfo
    {
        private string area, pref, city, title, forecastDay, 
            day, telop, imageUrl ;
        private int imageWidth, imageHeight;
        private DateTime forecastDate, publicTime;
        private Image weatherIcon;
        private double? tempMaxCelsius, tempMinCelsius;
        
        public string Area
        {
            get { return area; }
            set { area = value; }
        }

        public string Pref
        {
            get { return pref; }
            set { pref = value; }
        }

        public string City
        {
            get { return city; }
            set { city = value; }
        }

        public string Title
        {
            get { return title; }
            set { title = value; }
        }

        public string ForecastDay
        {
            get { return forecastDay; }
            set { forecastDay = value; }
        }

        public string Day
        {
            get { return day; }
            set { day = value; }
        }

        public DateTime ForecastDate
        {
            get { return forecastDate; }
            set { forecastDate = value; }
        }

        public DateTime PublicTime
        {
            get { return publicTime; }
            set { publicTime = value; }
        }

        public string Telop
        {
            get { return telop; }
            set { telop = value; }
        }

        public string ImageUrl
        {
            get { return imageUrl; }
            set {
                imageUrl = value;
                weatherIcon = null;
            }
        }

        public int ImageWidth
        {
            get { return imageWidth; }
            set { imageWidth = value; }
        }

        public int ImageHeight
        {
            get { return imageHeight; }
            set { imageHeight = value; }
        }

        public double? TempMaxCelsius
        {
            get { return tempMaxCelsius; }
            set { tempMaxCelsius = value; }
        }

        public double? TempMinCelsius
        {
            get { return tempMinCelsius; }
            set { tempMinCelsius = value; }
        }

        public Image WeatherIcon
        {
            get {
                if (weatherIcon != null)
                    return weatherIcon;
                else
                {
                    WebResponse iconResponse = null; 
                    try
                    {
                        iconResponse =
                            WebRequest.Create(ImageUrl).GetResponse();
                        Stream myStream = iconResponse.GetResponseStream();
                        weatherIcon = Image.FromStream(myStream);
                    }
                    catch
                    {
                        weatherIcon = null;
                    }
                    finally
                    {
                        if (iconResponse != null)
                            iconResponse.Close();
                    }
                    return weatherIcon;
                }
            }
        }
    }
}