using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using LiveSearch2.com.msn.search.soap;

namespace LiveSearch2
{
    public partial class Form1 : Form
    {
        private const int countPerPage = 50;
        private string appId;

        private MSNSearchService searchService;
        private SearchState currentState;

        public Form1()
        {
            InitializeComponent();
        }

        private void goButton_Click(object sender, EventArgs e)
        {
            if (currentState != null)
            {
                MessageBox.Show("sł", Text,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            listView1.Items.Clear();

            // 񓯊Ăяo
            searchService = new MSNSearchService();
            searchService.SearchCompleted += new SearchCompletedEventHandler(searchService_SearchCompleted);
            searchService.Timeout = 3000;

            // p[^

            string queryText = keywordTextBox.Text;

            if (jaCheckBox.Checked)
                queryText += " language:ja";

            SourceRequest sr = new SourceRequest();
            sr.Source = SourceType.Web;
            sr.Offset = 0;
            sr.Count = Math.Min(50, (int)numericUpDown1.Value);
            sr.ResultFields = ResultFieldMask.All;

            SearchRequest mySearchRequest = new SearchRequest();
            mySearchRequest.Requests = new SourceRequest[] { sr };
            mySearchRequest.Query = queryText;
            mySearchRequest.AppID = appId;
            mySearchRequest.CultureInfo = "ja-JP";

            // ̐ݒ

            SearchState searchState = new SearchState();

            searchState.TotalCount = 0;
            searchState.RequestCount = (int)numericUpDown1.Value;
            searchState.RequestData = mySearchRequest;

            currentState = searchState;

            // ̎s

            searchService.SearchAsync(mySearchRequest, searchState);

        }

        private void listView1_DoubleClick(object sender, EventArgs e)
        {
            if (listView1.SelectedItems.Count > 0)
            {
                Process.Start(((SearchResultListViewItem)listView1.SelectedItems[0]).Result.Url);
            }
        }

        private void cancelButton_Click(object sender, EventArgs e)
        {
            if (currentState != null)
                searchService.CancelAsync(currentState);
        }

        public void searchService_SearchCompleted(object sender, SearchCompletedEventArgs e)
        {

            if (e.Cancelled)
            {
                MessageBox.Show("̓LZ܂", Text,
                    MessageBoxButtons.OK, MessageBoxIcon.Information);
                currentState = null;
                progressBar1.Value = 0;
                return;
            }
            else if (e.Error != null)
            {
                MessageBox.Show(e.Error.Message, Text,
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                currentState = null;
                progressBar1.Value = 0;
                return;
            }

            // Xgr[Ɍʂ\

            SearchResponse searchResponse = e.Result;

            SourceResponse sourceResponse =
                searchResponse.Responses[0];

            Result[] sourceResults = sourceResponse.Results;

            listView1.BeginUpdate();

            foreach (Result sourceResult in sourceResults)
            {
                SearchResultListViewItem lvItem = new
                    SearchResultListViewItem(sourceResult);
                listView1.Items.Add(lvItem);
            }

            listView1.EndUpdate();

            // ēxsKv邩?

            SearchState searchState = (SearchState)(e.UserState);
            int goalTotal = Math.Min(searchState.RequestCount,
                sourceResponse.Total);
            int currentCount = Math.Min(sourceResponse.Total -
                sourceResponse.Offset,
                searchState.RequestData.Requests[0].Count);
            int currentTotal = searchState.TotalCount + currentCount;

            if (goalTotal > 0)
                progressBar1.Value = currentTotal * 100 / goalTotal;

            if (goalTotal > currentTotal)
            {
                searchState.TotalCount += currentCount;
                searchState.RequestData.Requests[0].Offset =
                    currentTotal;
                searchState.RequestData.Requests[0].Count =
                    Math.Min(goalTotal - currentTotal, countPerPage);

                searchService.SearchAsync(searchState.RequestData,
                    searchState);
            }
            else
            {
                currentState = null;
                progressBar1.Value = 0;
            }
        }

        private void settingButton_Click(object sender, EventArgs e)
        {
            using (Form2 idDialog = new Form2())
            {
                idDialog.ApplicationID = appId;

                if (idDialog.ShowDialog() == DialogResult.OK)
                {
                    appId = idDialog.ApplicationID;
                    Properties.Settings.Default.ApplicationId = appId;
                    Properties.Settings.Default.Save();
                }
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            appId = Properties.Settings.Default.ApplicationId;
        }

    }

    public class SearchResultListViewItem : ListViewItem
    {
        private Result innerResult;

        public SearchResultListViewItem(Result r)
        {
            innerResult = r;
            Text = r.Title;
            SubItems.Add(r.Description);
        }

        public Result Result
        {
            get { return innerResult; }
        }
    }

    public class SearchState
    {
        private int requestCount, totalCount;
        private SearchRequest requestData;

        public int RequestCount
        {
            get { return requestCount; }
            set { requestCount = value; }
        }

        public int TotalCount
        {
            get { return totalCount; }
            set { totalCount = value; }
        }

        public SearchRequest RequestData
        {
            get { return requestData; }
            set { requestData = value; }
        }
    }
}